/** ###################################################################
**     Filename  : ToneGenerate.c
**     Project   : DDSTest
**     Processor : MC56F8006_48_LQFP
**     Version   : Driver 01.13
**     Compiler  : Metrowerks DSP C Compiler
**     Date/Time : 7/24/2009, 4:32 PM
**     Abstract  :
**         Main module.
**         This module contains user's application code.
**     Settings  :
**     Contents  :
**         No public methods
**
**     (c) Copyright UNIS, a.s. 1997-2008
**     UNIS, a.s.
**     Jundrovska 33
**     624 00 Brno
**     Czech Republic
**     http      : www.processorexpert.com
**     mail      : info@processorexpert.com
** ###################################################################*/
/* MODULE ToneGenerate */


/* Including needed modules to compile this module/procedure */
#include "Cpu.h"
#include "AudioOutputPWM.h"
/* Including shared modules, which are used for whole project */
#include "PE_Types.h"
#include "PE_Error.h"
#include "PE_Const.h"
#include "IO_Map.h"

#include "DDS.h"
#include "ToneGenerate.h"

// constants for hack to reduce amplitude of single tones
#define SINGLE_TONE_SCALING_NUMERATOR		6
#define SINGLE_TONE_SCALING_DENOMINATOR	10

/* tone pair lookup tables */
typedef struct
{
	UWord16	LowtoneFrequency;
	UWord16 HightoneFrequency;
}	TONE_PAIR;

__pmem static const TONE_PAIR DTMFTonePairLookupTable[MAX_KEY_PRESS_VALUES] =
{
	// invalid keypress
	{0, 0},
	// '1' key
	{697, 1209},
	// '2' key
	{697, 1336},
	// '3' key
	{697, 1477},
	// '4' key
	{770, 1209},
	// '5' key
	{770, 1336},
	// '6' key
	{770, 1477},
	// '7' key
	{852, 1209},
	// '8' key
	{852, 1336},
	// '9' key
	{852, 1477},
	// '0' key
	{941, 1336},
	// '*' key
	{941, 1209},
	// '#' key
	{941, 1477},
	// 'A' key
	{697, 1633},
	// 'B' key
	{770, 1633},
	// 'C' key
	{852, 1633},
	// 'D' key
	{941, 1633}
};


__pmem static const TONE_PAIR MFTonePairLookupTable[MAX_KEY_PRESS_VALUES] =
{
	// invalid keypress
	{0, 0},
	// '1' key
	{700, 900},
	// '2' key
	{700, 1100},
	// '3' key
	{900, 1100},
	// '4' key
	{700, 1300},
	// '5' key
	{900, 1300},
	// '6' key
	{1100, 1300},
	// '7' key
	{700, 1500},
	// '8' key
	{900, 1500},
	// '9' key
	{1100, 1500},
	// '0' key
	{1300, 1500},
	// '*' key
	{1100, 1700},
	// '#' key
	{1500, 1700},
	// 'A' key
	{0, 0},
	// 'B' key
	{0, 0},
	// 'C' key
	{0, 0},
	// 'D' key
	{2600, 0}
};

__pmem static const TONE_PAIR RedboxTonePair =
{1700, 2200};


/* static variables */
static DDS_16 LowTone;
static DDS_16 HighTone;

static UWord8 LowToneNeedsInitialization = 1;
static UWord8 HighToneNeedsInitialization = 1;

void PlayTone(Keypress_t Keycode, KeytoneMode_t Mode, UWord16 SampleRate)
{
	TONE_PAIR TonePair = {0, 0};
	UWord16 PWMRatio = 32678;
	Word32 OutputBlend;
	
	if(KEYTONE_MODE_DTMF == Mode)
		TonePair = DTMFTonePairLookupTable[Keycode];
	else if(KEYTONE_MODE_REDBOX == Mode)
	{
		// special handling for redbox mode
		// since there is only a single tone pair, we use
		// any valid keypress as an 'on', and invalid keypress
		// (or no keypress) as an 'off'
		if(KEY_PRESS_INVALID != Keycode)
			TonePair = RedboxTonePair;
	}
	else
		TonePair = MFTonePairLookupTable[Keycode];
	
	if(0 == TonePair.LowtoneFrequency)
	{
		LowToneNeedsInitialization = 1;
	}
	else
	{
		if(1 == LowToneNeedsInitialization)
		{
			DDS16Initialize(&LowTone, SampleRate, TonePair.LowtoneFrequency, 0, DDS_SINE);
			LowToneNeedsInitialization = 0;
		}
		
		DDS16Update(&LowTone);
	}

	if(0 == TonePair.HightoneFrequency)
	{
		HighToneNeedsInitialization = 1;
	}
	else
	{
		if(1 == HighToneNeedsInitialization)
		{
			DDS16Initialize(&HighTone, SampleRate, TonePair.HightoneFrequency, 0, DDS_SINE);
			HighToneNeedsInitialization = 0;
		}
		
		DDS16Update(&HighTone);
	}
	
	if((0 != TonePair.LowtoneFrequency) && (0 != TonePair.HightoneFrequency))
	{
		OutputBlend = (LowTone.Output - 32768) / 2;
		OutputBlend += (HighTone.Output - 32768) / 2;
		OutputBlend += 32768;

		PWMRatio = (UWord16)OutputBlend;
	}
	else if(0 != TonePair.LowtoneFrequency)
	{
		// hack - reduce amplitude of single tones
		OutputBlend = LowTone.Output - 32768;
		OutputBlend *= SINGLE_TONE_SCALING_NUMERATOR;
		OutputBlend /= SINGLE_TONE_SCALING_DENOMINATOR;
		OutputBlend += 32768;
		
		PWMRatio = (UWord16)OutputBlend;
	}
	else if(0 != TonePair.HightoneFrequency)
	{
		// hack - reduce amplitude of single tones
		OutputBlend = HighTone.Output - 32768;
		OutputBlend *= SINGLE_TONE_SCALING_NUMERATOR;
		OutputBlend /= SINGLE_TONE_SCALING_DENOMINATOR;
		OutputBlend += 32768;
		
		PWMRatio = (UWord16)OutputBlend;
	}
	
	AudioOutputPWM_SetRatio16(PWMRatio);
	
}

/* END ToneGenerate */
